# ---------------------------------------------------------------------------
# k_circulant_n.sage
# Copyright (C) 2009 Minh Van Nguyen <nguyenminh2@gmail.com>
#
# This Sage script generates ring lattices, each with half the total number
# of edges removed. Such graphs can then be rewired as per Watts & Strogatz.
# Note that this script does not consider the problem of random edge
# rewiring. This script was written and tested using Sage 3.2.x. For more
# information about Sage, please visit www.sagemath.org. Before running this
# script, make sure that a directory named "networks-half-edges-r" exists
# in the current directory.
#
# REFERENCES:
# [1] V. Latora & M. Marchiori. Economic small-world behavior in weighted
#     networks. The European Physical Journal B, 32(2):249--263, 2003.
#
# [2] D. Watts & S.H. Strogatz. Collective dynamics of "small-world"
#     networks. Nature, 393(4):440--442, 1998.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# ---------------------------------------------------------------------------


def to_r(e, g):
    """
    Generates R code representation of the (n, k) ring lattice g.

    The (n, k) ring lattice g is assumed to have half the total number of
    its edges already removed. The R code matrix representation of g is an
    adjacency matrix and is written to a text file.

    INPUT:
        e -- a positive integer index used for name the R script
            containing R code matrix representation of g.
        g -- an (n, k) ring lattice with half of its total number of edges
            removed.

    OUTPUT:
        Write the R code matrix representation of g to an R script.
    """
    nVertices = g.order()
    outFile = open("networks-half-edges-r/graph-" + str(e) + ".r", "w")
    nrow = g.order()
    ncol = g.order()
    outFile.write("mat <- matrix(nrow = " + str(nrow)
                  + ", ncol = " + str(ncol) + ")\n")
    for i in xrange(nrow):
        row = str(g.adjacency_matrix()[i])
        row = "c" + row
        outFile.write("mat[" + str(i+1) + ",] <- " + row + "\n")
    outFile.close()

def remove_half_edges(n, k):
    """
    Randomly removes half the total number of edges from a k-circulant graph
    with n vertices.

    A k-circulant graph with n vertices is simply a ring lattice with n
    nodes, each of which is connected to its k neighbours. Such a graph
    is also referred to as an (n, k) ring lattice. Such random removal is
    used in "Model 4" of Latora & Marchiori [2].

    INPUT:
        n -- the number of vertices.
        k -- the number of per-vertex degree (must be an even integer).

    OUTPUT:
        An (n, k) ring lattice with half of the total number of its edges
        removed.
    """
    from sage.misc.prandom import choice

    adj = [a for a in xrange(1, k/2+1)]
    G = graphs.CirculantGraph(n, adj)

    # remove half the total number of edges from G
    nEdges = list(G.edge_iterator(labels = False))
    elimTotal = G.size() / 2
    elim = 0
    while elim < elimTotal:
        edge = choice(nEdges)
        G.delete_edge(edge[0], edge[1])
        while not G.is_connected():
            G.add_edge(edge[0], edge[1])
            edge = choice(nEdges)
            G.delete_edge(edge[0], edge[1])
        nEdges = list(G.edge_iterator(labels = False))
        elim += 1

    return G

# As used by Watts & Strogatz [1] and Latora & Marchiori [2].
n = 1000

# As used by Latora & Marchiori [2] in their "Model 4".
k = 6

# Also known as the number of rewiring probabilities. This number depends
# on how many rewiring probability points you want to use.
nTimes = 37

for i in xrange(nTimes):
    print "[%s] generating network" % (i+1)
    g = remove_half_edges(n, k)
    print "[%s] converting network to R code" % (i+1)
    to_r(i+1, g)
