# ---------------------------------------------------------------------------
# network-metrics-lm.r
# Copyright (C) 2008, 2009 -- Minh Van Nguyen <nguyenminh2@gmail.com>
#
# An R script to compute network metrics of Latora-Marchiori networks.
# That is, this script calculates the local and global efficiencies and
# network cost defined by Latora & Marchiori [1] as generalizations of
# the Watts-Strogatz [2] small world network metrics.
#
# REFERENCES:
# [1] V. Latora & M. Marchiori. Economic small-world behavior in weighted
#     networks. The European Physical Journal B, 32(2):249--263, 2003.
#
# [2] D. Watts & S.H. Strogatz. Collective dynamics of "small-world"
#     networks. Nature, 393(4):440--442, 1998.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
# ---------------------------------------------------------------------------


# Clear memory, removing (almost) everything in the working environment
# without any warning. Be careful with what you wish for.
rm(list = ls())
library("grDevices")  # for colours
library("sna")        # for social network analysis
# For various graph-theoretic operations, in particular, weighted shortest
# paths.
library("igraph")


### Functions


# The average efficiency E of a graph G, where the graph has N vertices and
# K edges. This function can also be used to calculate the average
# efficiency even if G is a complete graph on N vertices. Such a complete
# graph is also denoted G^{ideal}, i.e. the ideal case where G has all the
# possible
#
# N(N - 1) / 2
#
# edges. Then the global and local efficiencies are defined in terms of E(G)
# and E(G^{ideal}). These notions of efficiency of a graph are defined in
# the paper (Latora & Marchiori 2003).
#
# INPUT:
#    adjMat -- the adjacency matrix of the underlying graph.
#    weightedGeoMat -- a matrix of weighted geodesics, or weighted
#       shortest paths.
#    level -- whether the average efficiency returned would be used in
#       computing the local or global efficiency. The parameter level can
#       take on either of two string arguments: "global" to indicate that
#       the returned average efficiency is to be used in calculating the
#       global efficiency of a network; and "local" which signifies that
#       the returned average efficiency is to be used in calculating the
#       local efficiency of a network. Default is "global".
#
# OUTPUT:
#    the average efficiency E of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
averageEfficiency <- function(adjMat, weightedGeoMat) {
  # The number of vertices in weightedGeoMat. Thus adjMat and weightedGeoMat
  # are both N x N matrices.
  N <- dim(weightedGeoMat)[1]

  # for storing the average efficiency
  E <- NULL

  # check for the case that weightedGeoMat is a 1 x 1 matrix
  if (N == 1) {
    # harmonicSum <- 0
    E <- 0
  }
  else {
    # Compute the harmonic sum of lower triangular matrix of weightedGeoMat,
    # excluding the main diagonal.
    colLimit <- 0
    colStart <- 1
    rowStart <- 2
    harmonicSum <- 0
    for (row in rowStart:N) {
      colLimit <- colLimit + 1
      for (col in colStart:colLimit) {
        # Avoid the case where there's no path between vertices i and j. If
        # no path exists between i and j, then d_ij = +oo, which is positive
        # infinity. As d_ij -> +oo, then (1 / d_ij) -> 0. If adjMat
        # represents a totally isolated graph G, then the average efficiency
        # of G is 0.
        if (weightedGeoMat[row, col] != "Inf") {
          harmonicSum <- harmonicSum + (1 / weightedGeoMat[row, col])
        }
      }
    }

    # compute average efficiency
    E <- harmonicSum / (N * (N - 1))
  }

  E
}


# The global efficiency E_glob of a graph G, where the graph has N vertices
# and K edges. The notion of global efficiency of a graph is defined in the
# paper (Latora & Marchiori 2003). See also the function averageEfficiency,
# which defines the average efficiency of G. The measure E_glob is defined
# as
#
# E_glob = E(G) / E(G^{ideal})
#
# where G^{ideal} is the complete graph on N vertices. Thus E_glob is a
# ratio of the average efficiencies of two types of graphs: (1) the average
# efficiency of G itself; (2) the average efficiency of the complete graph
# on N vertices, which is the number of vertices in G.
#
# INPUT:
#    adjMat -- an N x N adjacency matrix of G.
#    weightMat -- an N x N weight matrix of G. See the function weightMatrix
#       for further details.
#
# OUTPUT:
#    the global efficiency E_glob of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
globalEfficiency <- function(adjMat, weightMat) {
  # The number of vertices in the underlying graph G. Thus the column and
  # row dimensions must be equal.
  N <- dim(adjMat)[1]

  # matrix of weighted geodesics for G
  weightedGeoMat <- weightedGeodesics(adjMat, weightMat)
  # average efficiency of G
  aveEfficiency <- averageEfficiency(adjMat, weightedGeoMat)

  # Construct the adjacency matrix of a complete graph on N vertices. By
  # definition of complete graphs, a complete graph K_n and its geodesic
  # matrix G_dist are equivalent. That is, K_n and G_dist are copies of
  # each other. Also, G_dist has 1 everywhere, and 0 on the main diagonal.
  gIdealMat <- matrix(1, nrow = N, ncol = N)
  for (i in 1:N) {
    gIdealMat[i, i] <- 0
  }

  # matrix of weighted geodesics for G^{ideal}
  weightedGeoMat <- weightedGeodesics(gIdealMat, weightMat)
  # average efficiency of G^{ideal}
  aveEfficiencyGIdeal <- averageEfficiency(gIdealMat, weightedGeoMat)

  # compute the global efficiency
  Eglob <- aveEfficiency / aveEfficiencyGIdeal
}


# Let adjMat be an adjacency matrix of an undirected graph G. For a given
# vertex i of G, find the indices of the immediate neighbours of i.
#
# INPUT:
#    adjMat -- an adjacency matrix of an undirected graph G.
#    i -- a vertex of G.
#
# OUTPUT:
#    a vector containing vertices that are immediate neighbours of i.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
immediateNeighbours <- function(adjMat, i) {
  # The column dimension of adjMat. As adjMat is an adjacency matrix, it
  # doesn't matter if we get either of its row or column dimensions.
  colNum <- dim(adjMat)[2]

  # for storing indices of the immediate neighbours of vertex i
  neighIndex <- c()

  # find indices of the immediate neighbours of vertex i
  for (col in 1:colNum) {
    if (adjMat[i, col] == 1) {
      neighIndex <- c(neighIndex, col)
    }
  }

  neighIndex
}


# The adjacency matrix of G_i. If G is an undirected graph and i is a
# vertex of G, then G_i is the subgraph of neighbours of i, excluding i
# itself.
#
# INPUT:
#    aMat -- the adjacency matrix of the graph G.
#    i -- the index of the vertex whose neighbours we want to consider.
#       Let r and c be the row and column dimensions of aMat, respectively.
#       Then 1 < i < r or 1 < i < c.
#
# OUTPUT:
#    an adjacency matrix of G_i. If i is an isolated vertex, then return
#    an n x n zero matrix. If i is not isolated but all vertices in G_i are
#    isolated from each other, then return an n x n zero matrix. Else we know
#    that i is not isolated and there is a pair of vertices in G_i that
#    is connected by an edge; in this case, return an n x n matrix where
#    n > 0.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
neighboursAdjMat <- function(aMat, i) {
  # The column dimension of aMat. As aMat is an adjacency matrix, it
  # doesn't matter if we get either of its row or column dimensions.
  colNum <- dim(aMat)[2]

  # an adjacency matrix of the neighbours of vertex i
  neighAMat <- NULL

  # for storing indices of the immediate neighbours of vertex i
  neighIndex <- NULL

  # check if i is an isolated vertex
  if (sum(aMat[i, ]) == 0) {
    # If i is an isolated vertex, then return an n x n zero matrix, which
    # is of the same dimensions as those of aMat.
    neighAMat <- matrix(0, nrow = dim(aMat)[1], ncol = dim(aMat)[2])
  }
  # now we know that i is connected to at least another vertex
  else {
    # get indices of the immediate neighbours of i
    neighIndex <- immediateNeighbours(aMat, i)

    # The variables neighRowIndex and neighColIndex should be vectors of
    # equal length. Let neighRowIndex be of length n, then neighColIndex
    # also has length n. For k = 1,...,n, neighRowIndex[k] and
    # neighColIndex[k] refer to vertices that are immediate neighbours of
    # vertex i, and such that neighRowIndex[k] and neighColIndex[k] are
    # connected by an (undirected) edge.
    neighRowIndex <- NULL
    neighColIndex <- NULL
    for (row in 1:length(neighIndex)) {
      for (col in row:length(neighIndex)) {
        if (aMat[neighIndex[row], neighIndex[col]] == 1) {
          neighRowIndex <- c(neighRowIndex, neighIndex[row])
          neighColIndex <- c(neighColIndex, neighIndex[col])
        }
      }
    }

    # If i is not an isolated vertex, then the length of the vector
    # neighIndex is > 0. Let G_i be the subgraph of the neighbours of i. If
    # all vertices of G_i are isolated, then each of the vectors
    # neighRowIndex and neighColIndex has a length of zero. In this case,
    # neighAMat is a 2 x 0 matrix.
    neighAMat <- matrix(0, nrow = 2, ncol = length(neighRowIndex))
    neighAMat[1,] <- neighRowIndex
    neighAMat[2,] <- neighColIndex

    # check if G_i is totally isolated
    if (dim(neighAMat)[2] == 0) {
      # If G_i is totally isolated, then return an n x n zero matrix, which
      # is of the same dimensions as those of aMat.
      neighAMat <- matrix(0, nrow = dim(aMat)[1], ncol = dim(aMat)[2])
    }
    # now we know that at least one pair of vertices in G_i are connected
    else {
      neighAdjMat <- matrix(0, nrow = dim(aMat)[1], ncol = dim(aMat)[2])
      for (col in 1:dim(neighAMat)[2]) {
        neighAdjMat[neighAMat[1, col], neighAMat[2, col]] <- 1
        neighAdjMat[neighAMat[2, col], neighAMat[1, col]] <-  1
      }
      neighAMat <- neighAdjMat
    }
  }

  neighAMat
}


# The local efficiency E_loc of a graph G, where the graph has N vertices
# and K edges. The notion of local efficiency of a graph is defined in the
# paper (Latora & Marchiori 2003). See also the function averageEfficiency,
# which defines the average efficiency of G. The measure E_loc is defined
# as
#
# E_loc = (1/N) \sum_{i \in G} E(G_i) / E(G^{ideal}_i)
#
# where G_i is the subgraph of neighbours of vertex i, and G^{ideal}_i is
# the complete graph on N_i, which is the number of vertices in G_i. Note
# that G_i excludes the vertex i, and only considers the graph formed by
# its immediate neighbours.
#
# INPUT:
#    aMat -- the adjacency matrix of the graph G. If G is an undirected
#       graph, then aMat is symmetric about the main diagonal.
#
# OUTPUT:
#    the local efficiency E_loc of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
localEfficiency <- function(aMat, weightMat) {
  # The number of vertices in the underlying graph G. Thus the column and
  # row dimensions must be equal.
  N <- dim(aMat)[1]

  # summing the ratios (EGi / EIdealGi) for all vertices i
  cumSum <- 0           # the cumulative sum
  EGi <- 0              # average efficiency of G_i
  EIdealGi <- 0         # average efficiency of G^{ideal}_i
  idealGi <- 0          # adjacency matrix of G^{ideal}_i
  for (i in 1:N) {
    # adjacency matrix of G_i
    neighI <- neighboursAdjMat(aMat, i)
    # matrix of weighted geodesics for G_i
    weightedGeoMat <- weightedGeodesics(neighI, weightMat)
    # average efficiency of G_i
    EGi <- averageEfficiency(neighI, weightedGeoMat)

    # Construct the adjacency matrix of a complete graph on K_i vertices. By
    # definition of complete graphs, a complete graph K_n and its geodesic
    # matrix G_dist are equivalent, provided that K_n is unweighted.
    idealGi <- matrix(0, nrow = N, ncol = N)
    neighIndex <- immediateNeighbours(aMat, i)
    for (j in 1:length(neighIndex)) {
      for (k in j:length(neighIndex)) {
        idealGi[neighIndex[j], neighIndex[k]] <- 1
        idealGi[neighIndex[k], neighIndex[j]] <- 1
      }
      # do this since we want zeros along the main diagonal
      idealGi[neighIndex[j], neighIndex[j]] <- 0
    }

    # matrix of weighted geodesics for G^{ideal}_i
    weightedGeoMat <- weightedGeodesics(idealGi, weightMat)
    # average efficiency of G^{ideal}_i
    EIdealGi <- averageEfficiency(idealGi, weightedGeoMat)

    # Prevent division by zero, which is possible when EIdealGi = 0. If
    # both EGi and EIdealGi are zero, then we get (0 / 0), which returns
    # a NaN for "not a number". CAUTION: we need to consider four cases:
    #
    # EGi  EIdealGi
    # -------------
    # 0    0       <- (EGi / EIdealGi) = 0 because we say so
    # 0    y1      <- (EGi / EIdealGi) = 0
    # x1   0       <- Is it possible to get this case?
    # x2   y2      <- (EGi / EIdealGi) \in RR\{0}
    #
    # where x1, x2, y1, y2 \in RR are non-zero and RR is the set of
    # real numbers.
    if (EGi == 0) {
      cumSum <- cumSum + 0
    }
    else {
      cumSum <- cumSum + (EGi / EIdealGi)
    }
  }

  Eloc <- cumSum / N
}


# The cost of a network G with N vertices and K edges. For now, we assume
# that G is an undirected graph so that its adjacency matrix is symmetric
# about the main diagonal. The generalization of the Watts-Strogatz model
# contained in (Latora & Marchiori 2003) considers directed as well as
# undirected graphs.
#
# INPUT:
#    adjMat -- the adjacency matrix of G. This adjacency matrix must have
#       the same dimensions as the matrix of distances of G.
#    distMat -- the matrix of distances between pairs of vertices. This
#       distance matrix has the same dimensions as the adjacency matrix of
#       G.
#
# OUTPUT:
#    the cost of the network G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
networkCost <- function(adjMat, distMat) {
  netCost <- 0
  numerator <- 0
  denominator <- 0
  N <- dim(adjMat)[1]

  for (row in 2:N) {
    for (col in 1:(row - 1)) {
      numerator <- numerator + (adjMat[row, col] * distMat[row, col])
      denominator <- denominator + (distMat[row, col])
    }
  }

  netCost <- numerator / denominator
}


# The weight matrix of a ring lattice G that has N vertices. This weight is
# defined in terms of the Euclidean distance between pairs of nodes. If i
# and j are vertices of G, then the distance between i and j is
#
# l_ij = [2 * sin(|i - j| pi / N)] / [2 * sin(pi / N)]
#      = sin(|i - j| pi / N) / sin(pi / N)
#
# The distance between each pair of neighbouring vertices is l_ij = 1 and
# the distance from i to itself is trivially l_ii = 0. The weight matrix
# of G is denoted {l_ij}, which has zero along the main diagonal and is
# symmetric about this diagonal.
#
# INPUT:
#    n -- an integer > 0; this is the number of vertices of the ring
#       lattice G
#
# OUTPUT:
#    the weight matrix {l_ij} of G. If n <= 0, then return NULL.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
weightMatrix <- function(n) {
  weightMat <- NULL

  if (n > 0) {
    # construct an n x n matrix with zero everywhere
    weightMat <- matrix(0, nrow = n, ncol = n)

    # Calculate the Euclidean distances on the ring lattice. Perhaps we
    # need only to consider either of the lower triangular or upper
    # triangular matrices, excluding the main diagonal.
    for (row in 1:n) {
      for (col in 1:n) {
        if (row != col) {
          # numerator <- 2 * sin((abs(row - col) * pi) / n)
          # denominator <- 2 * sin(pi / n)
          numerator <- sin((abs(row - col) * pi) / n)
          denominator <- sin(pi / n)
          weightMat[row, col] <- numerator / denominator
        }
      }
    }
  } else {
    weightMat <- NULL
  }

  weightMat
}


# A matrix of weighted shortest paths for a weighted ring lattice G. The
# lattice G has N vertices and a degree of k per vertex.
#
# INPUT:
#    adjMat -- the adjacency matrix of G. If G is undirected, then adjMat
#       is symmetric about the main diagonal. The adjacency matrix of G
#       must have the same dimensions as the weight matrix of G.
#    weightMat -- a matrix of edge weights. This is an N x N matrix,
#       where N is the number of vertices in G. If G is undirected, then
#       weightMat is symmetric about the main diagonal. The adjacency
#       matrix of G must have the same dimensions as the weight matrix of G.
#
# OUTPUT:
#    an N x N matrix of weighted shortest paths. If G is totally isolated,
#    then return an N x N matrix with +oo everywhere, and zero along the
#    main diagonal. Else G has an (undirected) edge connecting a pair of
#    its vertices, so we return an N x N matrix of weighted shortest paths.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
weightedGeodesics <- function(adjMat, weightMat) {
  N <- dim(adjMat)[1]
  weightedGeo <- NULL

  # Check for totally isolated graphs. The graph G represented by adjMat is
  # totally isolated if all its vertices are isolated from each other. For
  # a totally isolated graph G of dimensions N x N, its corresponding
  # matrix of weighted geodesics is an N x N matrix with +oo (positive
  # infinity) everywhere, and zero along the main diagonal.
  if (sum(adjMat) == 0) {
    weightedGeo <- matrix(Inf, nrow = N, ncol = N)
    for (i in 1:N) {
      weightedGeo[i, i] <- 0
    }
  }
  # Now we know that G is not totally isolated, so at least one pair of
  # vertices in G is connected by an (undirected) edge. Then proceed to
  # find the matrix of weighted geodesics corresponding to G.
  else {
    colLimit <- 0
    colStart <- 1
    rowStart <- 2
    startVertex <- c()
    endVertex <- c()
    edgeWeight <- c()

    # As weightMat is symmetric about the main diagonal, we only need to
    # consider its lower (or upper) triangular matrix, excluding entries
    # along the main diagonal.
    for (row in rowStart:N) {
      colLimit <- colLimit + 1
      for (col in colStart:colLimit) {
        if (adjMat[row, col] == 1) {
          startVertex <- c(startVertex, row)
          endVertex <- c(endVertex, col)
          edgeWeight <- c(edgeWeight, weightMat[row, col])
        }
      }
    }

    e <- c()
    for (i in 1:length(startVertex)) {
      e <- c(e, startVertex[i], endVertex[i], edgeWeight[i])
    }
    emat <- matrix(nc = 3, byrow = TRUE, e)
    for (row in 1:dim(emat)[1]) {
      emat[row, 1] <- emat[row, 1] - 1
      emat[row, 2] <- emat[row, 2] - 1
    }
    g <- add.edges(graph.empty(N, directed = FALSE),
                   t(emat[, 1:2]), weight = emat[, 3])
    weightedGeo <- shortest.paths(g)
  }

  weightedGeo
}


# The main routine. This is where the network metrics are calculated. For
# the Watts-Strogatz model, the network metrics is comprised of the
# characteristic path length L and the clustering coefficient C. As regards
# the generalization of Watts-Strogatz contained in the paper
# (Latora & Marchiori 2003), the network metrics are the local efficiency
# E_loc, the global efficiency E_glob, and the network cost C_G.
#
# INPUT:
#    regmat -- a regular matrix.
#    probabilities -- a set of re-wiring probabilities.
#    weightMat -- a matrix of edge weights.
#    experiment -- n-th experiment
#
# OUTPUT:
#     Network metrics using the measures described in Latora & Marchiori [1].
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
calculateNetworks <- function(regmat, probabilities, weightMat, experiment) {
  # set of adjacency matrices
  nets <- array(NA, dim = c(length(probabilities) + 1, n, n))
  # first matrix is regular matrix
  nets[1,,] <- regmat[,]

  # read in rewired networks for the specified experiment number
  print("read in rewired networks")
  counter <- 1
  for (p in 1:length(probabilities)) {
    # Read in rewired network into memory and the resulting object is named
    # "mat".
    source(paste("networks-r/graph-", experiment, "-", p, ".r", sep = ""))
    nets[counter + 1,,] <- mat[,]
    counter <- counter + 1
  }

  # This section is for the Latora-Marchiori generalization.
  # Global and local efficiencies, and network cost
  Eglob <- NULL
  Eloc <- NULL
  netCost <- NULL
  for (counter in 1:(length(probabilities) + 1)) {
    print(c("metrics for n-th rewiring probability ", counter))
    Eglob[counter] <- globalEfficiency(nets[counter,,], weightMat)
    Eloc[counter] <- localEfficiency(nets[counter,,], weightMat)
    netCost[counter] <- networkCost(nets[counter,,],
                                    weightedGeodesics(nets[counter,,],
                                                      weightMat))
  }

  # the structure to return
  result <- cbind(Eglob[1] / Eglob,
                  Eloc / Eloc[1],
                  netCost / netCost[1])
}


# Simulates a circular list. We are only interested in the index i of a
# member of this list, which has n members. One way to conceptualize this
# list is to visualize all n members as arranged in a cycle graph, in
# which each member i has an edge connecting it to i + 1, and an edge
# connecting it to i - 1. Another way to think about this function is to
# interpret it as a simple implementation of the group Z/Zn, where only
# the index of each i in Z/Zn is returned.
#
# INPUT:
#    index -- integer; index of an element in this circular list.
#    length -- integer > 0; the number of elements in this circular list.
#       FIXME: Maybe it's a good idea to implement the case where
#       length < 0, or provide some sanity checking to take care of that
#       possibility.
#
# OUTPUT:
#    If 0 < index <= length, then return index. If index > length, then
#    return index mod length. Else index < 0, so return index mod length.
#    If length is 0, then return NaN (not a number).
#
# AUTHOR:
#    Rodolfo Garcia-Flores
#    Documentation by Minh Van Nguyen <nguyenminh2@gmail.com>
#
returnIndex <- function(index, length) {
  if ((index > 0) && (index <= length)) {
    index
  }
  else if (index > length) {
    index - length * floor(index / length)
  }
  else {
    (length + index) + length * floor(-1 * index / length)
  }
}


### Main script


# Actual values should be n = 1000, k = 10. When run with these values,
# the script should take a few hours to complete. Test values can be
# n = 20, k = 4
n <- 1000             # vertices
k <- 6                # edges per vertex, MUST BE EVEN.
nTimes <- 20          # should be 20

# data for a logarithmically-scaled probability vector
nPoints <- 37
minProb <- 1e-4
maxProb <- 1

# output file name
summaryFileName <- "small-world-summary.txt"

# connectivity matrix of a regular network with no loops
regularMatrix <- matrix(0, nrow = n, ncol = n)
for (i in 1:n) {
  for (index in (k/2):1) {
    # Get right the indexes.
    jplus  <- returnIndex(i + index, n)
    jminus <- returnIndex(i - index, n)
    regularMatrix[i, jplus] <- regularMatrix[i, jminus] <- 1
  }
}

# logarithmically-scaled probability vector
factor <- (maxProb / minProb)^(1 / (nPoints - 1))
probs <- NULL
for (pt in 1:nPoints) {
  probs[pt] <- minProb * factor^(pt - 1)
}

# variable aliases
numMeasures <- 3   # how many measures
GE <- 1            # global efficiency
LE <- 2            # local efficiency
NC <- 3            # network cost

# weight matrix of the ring lattice
weightMat <- weightMatrix(n)

# first probability is zero, i.e. the regular matrix
results <- array(NA, dim = c(nTimes, length(probs) + 1, numMeasures))

for (experiment in 1:4) {
  print(c("network metrics for experiment ", experiment))
  results[experiment,,] <- calculateNetworks(regularMatrix,
                                             probs,
                                             weightMat,
                                             experiment)
}

# averages
GEmeans <- colSums(results[,,GE]) / nTimes
LEmeans <- colSums(results[,,LE]) / nTimes
NCmeans <- colSums(results[,,NC]) / nTimes

# write table of results
summary <- cbind(c(0, probs), GEmeans, LEmeans, NCmeans)
write.table(summary, file = summaryFileName, sep = "\t")

# plot variables
# insert plotting code here
