# ---------------------------------------------------------------------------
# rewire-lattices.r
# Copyright (C) 2009 Minh Van Nguyen <nguyenminh2@gmail.com>
#
# This R script can be used to rewire (n, k) ring lattices that have had
# 50 percent of their total number of edges removed. The rewiring process
# is per Watts & Strogatz [2]. The resulting rewired networks are used
# in "Model 4" of Latora & Marchiori [1]. Before running this script, make
# sure that a directory named "networks-dat" exists in the current directory.
# For more information about R, please visit www.r-project.org.
#
# REFERENCES:
# [1] V. Latora & M. Marchiori. Economic small-world behavior in weighted
#     networks. The European Physical Journal B, 32(2):249--263, 2003.
#
# [2] D. Watts & S.H. Strogatz. Collective dynamics of "small-world"
#     networks. Nature, 393(4):440--442, 1998.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# ---------------------------------------------------------------------------


# Some housekeeping before generating random Latora-Marchiori networks.
# Clear memory, removing (almost) everything in the working environment
# without any warning. Be careful with what you wish for.
rm(list = ls())
# For social network analysis
library("sna")
# For various graph-theoretic operations, in particular, weighted shortest
# paths.
library("igraph")


### Functions


# Simulates a circular list. We are only interested in the index i of a
# member of this list, which has n members. One way to conceptualize this
# list is to visualize all n members as arranged in a cycle graph, in
# which each member i has an edge connecting it to i + 1, and an edge
# connecting it to i - 1. Another way to think about this function is to
# interpret it as a simple implementation of the group Z/Zn, where only
# the index of each i in Z/Zn is returned.
#
# INPUT:
#    index -- integer; index of an element in this circular list.
#    length -- integer > 0; the number of elements in this circular list.
#       FIXME: Maybe it's a good idea to implement the case where
#       length < 0, or provide some sanity checking to take care of that
#       possibility.
#
# OUTPUT:
#    If 0 < index <= length, then return index. If index > length, then
#    return index mod length. Else index < 0, so return index mod length.
#    If length is 0, then return NaN (not a number).
#
# AUTHOR:
#    Rodolfo Garcia-Flores
#    Documentation by Minh Van Nguyen <nguyenminh2@gmail.com>
#
returnIndex <- function(index, length) {
  if ((index > 0) && (index <= length)) {
    index
  }
  else if (index > length) {
    index - length * floor(index / length)
  }
  else {
    (length + index) + length * floor(-1 * index / length)
  }
}


# The rewiring routine.
#
# INPUT:
#    aMatrix -- an adjacency matrix.
#    aProbability -- double; a probability value p such that 0 < p < 1. This
#       value determines the probability that an edge incident on a vertex
#       is re-wired.
#
# OUTPUT:
#    An adjacency matrix with a number of the vertices re-wired.
#
# AUTHOR:
#    Rodolfo Garcia-Flores
#    Documentation by Minh Van Nguyen <nguyenminh2@gmail.com>
#
rewire <- function(aMatrix, aProbability) {
  currentMat <- aMatrix
  for (i in 1:n) {
    for (j in 1:i) {
      if ((currentMat[i, j] != 0) && (runif(1) < aProbability)) {
        # To vertices different to i and
        # different to those already connected,
        # preferably to nodes that are isolated.
        # This should prevent having isolated regions.
        isolatedNodes <- c(1:n)[colSums(currentMat[,]) == 0]
        nodesAlreadyConnected <- c(1:n)[currentMat[i,] > 0]
        excludedNodes <- c(i, nodesAlreadyConnected)
        notExcludedNodes <- (1:n)[-excludedNodes]

        # A list whose first elements are isolated nodes, the rest are
        # shuffled, not-excluded values.
        validPrioritisedNodes <- c(isolatedNodes,
			      	   sample(setdiff(notExcludedNodes,
						  isolatedNodes)))

        # Take first node index in list.
        newVertex <- validPrioritisedNodes[1]
        currentMat[i, j] <- 0
        currentMat[j, i] <- 0
        currentMat[i, newVertex] <- 1
        currentMat[newVertex, i] <- 1
      }
    }
  }

  currentMat
}


# The main routine. This is where the network metrics are calculated. For
# the Watts-Strogatz model, the network metrics is comprised of the
# characteristic path length L and the clustering coefficient C. As regards
# the generalization of Watts-Strogatz contained in the paper
# (Latora & Marchiori 2003), the network metrics are the local efficiency
# E_loc, the global efficiency E_glob, and the network cost C_G.
#
# INPUT:
#    regmat -- a regular matrix.
#    probabilities -- a set of re-wiring probabilities.
#
# OUTPUT:
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
latoraMarchioriGraphs <- function(regMat, probs, experiment) {
  for (p in 1:length(probs)) {
    # Reads in an (n, k) ring lattice which has 50 percent of its total
    # number of edges removed. The matrix is read into memory and named "mat".
    source(paste("networks-half-edges-r/graph-",
                 experiment, "-", p, ".r", sep = ""))

    # rewire with p-th probability
    print(c("rewiring with probability ", probs[p]))
    rewiredMat <- rewire(mat, probs[p])
    while (connectedness(rewiredMat) < 1) {
      rewiredMat <- rewire(regMat, probs[p])
    }
    # The number 1000 refers both to the column and row dimensions of the
    # Latora-Marchiori network.
    write.table(1000, file = paste("networks-dat/graph-",
                        experiment, "-", p, ".dat", sep = ""),
                row.names = FALSE, col.names = FALSE)
    write.table(1000, file = paste("networks-dat/graph-",
                        experiment, "-", p, ".dat", sep = ""),
                row.names = FALSE, col.names = FALSE, append = TRUE)
    write.table(probs[p], file = paste("networks-dat/graph-",
                            experiment, "-", p, ".dat", sep = ""),
                row.names = FALSE, col.names = FALSE, append = TRUE)
    write.table(rewiredMat, file = paste("networks-dat/graph-",
                              experiment, "-", p, ".dat", sep = ""),
                row.names = FALSE, col.names = FALSE, append = TRUE)
  }
}


### Start generate Latora-Marchiori networks here

# Experimental parameters.
# Actual values should be n = 1000, k = 10 or k = 6. When run with these
# values, the script should take a few hours to complete. Test values can be
# n = 20, k = 4
n <- 1000             # vertices
k <- 6                # edges per vertex, MUST BE EVEN.
nTimes <- 20          # should be 20

# data for a logarithmically-scaled probability vector
numPoints <- 37
minProb <- 1e-4
maxProb <- 1


# logarithmically-scaled probability vector
factor <- (maxProb / minProb)^(1 / (numPoints - 1))
probs <- NULL
for (pt in 1:numPoints) {
  probs[pt] <- minProb * factor^(pt - 1)
}

# Create rewired networks. The rewired networks are written to text files.
for (experiment in 1:nTimes) {
  print(c("Executing experiment ", experiment))
  latoraMarchioriGraphs(matHalfEdges, probs, experiment)
}
