# sw-latora-marchiori.r -- modelling small-world networks
# Copyright (C) 2008 Minh Van Nguyen <nguyenminh2@gmail.com>
#
# An R script to model small-world networks. This implements techniques
# in the paper:
#
# V. Latora & M. Marchiori. Economic small-world behavior in weighted
# networks. The European Physical Journal B, 32(2):249--263, 2003.
#
# which generalizes the approach described in:
#
# D. Watts & S.H. Strogatz. Collective dynamics of "small-world"
# networks. Nature, 393(4):440--442, 1998.
#
# Rodolfo Garcia-Flores has written an R script that implements the
# Watts-Strogatz model described in (Watts & Strogatz 1998).
#
# Minh Van Nguyen extended Rodolfo's code based on a generalization
# of the Watts-Strogatz model as detailed in the paper
# (Latora & Marchiori 2003).
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# Clear memory, removing (almost) everything in the working environment
# without any warning. Be careful with what you wish for.
rm(list = ls())
# For colours and fonts
library(grDevices)
# For fit, simulate and diagnose exponential-family models for networks
library("ergm")
# For social network analysis
library("sna")
# library("igraph")


# ===================================================================
# 1. SETUP (DATA, MODELS AND FUNCTIONS)
# ===================================================================


### Problem data


# Actual values should be n = 1000, k = 10. When run with these values,
# the script should take a few hours to complete. Test values can be
# n = 20, k = 4
n <- 1000            # vertices
k <- 10              # edges per vertex, MUST BE EVEN.
timesToRepeat <- 20  # should be 20

# Data for a logarithmically-scaled probability vector.
numberOfPoints <- 37
minProb <- 1e-4
maxProb <- 1

# Directories and file names
# Subdirectory names to organise I/O.
input <- "input"
output <- "output"

# Files in input directory to search for.
# Output file names
summaryFileName <- "small-world-summary.txt"

# Platform-specific directory separator.
slash <- .Platform$file.sep

# (Relative) subdirectory paths.
outputDir <- paste(".", slash, output, slash, sep = "")
inputDir  <- paste(".", slash, input, slash, sep = "")


### Functions


# Simulates a circular list. We are only interested in the index i of a
# member of this list, which has n members. One way to conceptualize this
# list is to visualize all n members as arranged in a cycle graph, in
# which each member i has an edge connecting it to i + 1, and an edge
# connecting it to i - 1. Another way to think about this function is to
# interpret it as a simple implementation of the group Z/Zn, where only
# the index of each i in Z/Zn is returned.
#
# INPUT:
#    index -- integer; index of an element in this circular list.
#    length -- integer > 0; the number of elements in this circular list.
#       FIXME: Maybe it's a good idea to implement the case where
#       length < 0, or provide some sanity checking to take care of that
#       possibility.
#
# OUTPUT:
#    If 0 < index <= length, then return index. If index > length, then
#    return index mod length. Else index < 0, so return index mod length.
#    If length is 0, then return NaN (not a number).
#
# AUTHOR:
#     Rodolfo Garcia-Flores
#     Documentation by Minh Van Nguyen <nguyenminh2@gmail.com>
#
returnIndex <- function(index, length) {
  if ((index > 0) && (index <= length)) {
    index
  }
  else if (index > length) {
    index - length * floor(index / length)
  }
  else {
    (length + index) + length * floor(-1 * index / length)
  }
}

# The re-wiring routine.
#
# INPUT:
#    aMatrix -- an adjacency matrix.
#    aProbability -- double; a probability value p such that 0 < p < 1. This
#       value determines the probability that an edge incident on a vertex
#       is re-wired.
#
# OUTPUT:
#    An adjacency matrix with a number of the vertices re-wired.
#
# AUTHOR:
#     Rodolfo Garcia-Flores
#     Documentation by Minh Van Nguyen <nguyenminh2@gmail.com>
#
reWire <- function(aMatrix, aProbability) {
  currentMat <- aMatrix
  for (i in 1:n) {
    for (j in 1:i) {
      if ((currentMat[i,j] != 0) && (runif(1) < aProbability)) {
        # To vertices different to i and
        # different to those already connected,
        # preferably to nodes that are isolated.
        # This should prevent having isolated regions.
        isolatedNodes <- c(1:n)[colSums(currentMat[,]) == 0]
        nodesAlreadyConnected <- c(1:n)[currentMat[i,] > 0]
        excludedNodes <- c(i, nodesAlreadyConnected)
        notExcludedNodes <- (1:n)[-excludedNodes]

        # A list whose first elements are isolated nodes, the rest are
        # shuffled, not-excluded values.
        validPrioritisedNodes <- c(isolatedNodes,
			      	   sample(setdiff(notExcludedNodes,
						  isolatedNodes)))

        # Take first node index in list.
        newVertex <- validPrioritisedNodes[1]
        currentMat[i,j] <- 0
        currentMat[j,i] <- 0
        currentMat[i, newVertex] <- 1
        currentMat[newVertex, i] <- 1
      }
    }
  }
  currentMat
}

# The average efficiency E of a graph G, where the graph has N vertices and
# K edges. This function can also be used to calculate the average
# efficiency even if G is a complete graph on N vertices. Such a complete
# graph is also denoted G^{ideal}, i.e. the ideal case where G has all the
# possible
#
# N(N - 1) / 2
#
# edges. Then the global and local efficiencies are defined in terms of E(G)
# and E(G^{ideal}). These notions of efficiency of a graph are defined in
# the paper (Latora & Marchiori 2003).
#
# INPUT:
#    geodesicMat -- the matrix of the shortest path lengths between pairs of
#       vertices in G. If i and j are vertices of G, then d_ij denotes the
#       shortest path length between i and j. If geodesicMat describes the
#       geodesics of pairs of vertices in an undirected graph, then
#       geodesicMat is symmetric about the main diagonal. Note that
#       geodesicMat must be a square matrix, so that its row and column
#       dimensions both equal the number of vertices in the underlying
#       graph.
#
# OUTPUT:
#    the average efficiency E of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
averageEfficiency <- function(geodesicMat) {
  # the number of vertices in geodesicMat
  N <- dim(geodesicMat)[1]
  E <- NULL

  # check for the case that geodesicMat is a 1 x 1 matrix
  if (N == 1) {
    # harmonicSum <- 0
    E <- 0
  }
  else {
    # Compute the harmonic sum of lower triangular matrix of geodesicMat,
    # excluding the main diagonal.
    colLimit <- 0
    colStart <- 1
    rowStart <- 2
    harmonicSum <- 0
    for (row in rowStart:N) {
      colLimit <- colLimit + 1
      for (col in colStart:colLimit) {
        # Avoid the case where there's no path between vertices i and j. If
        # no path exists between i and j, then d_ij = +oo, which is positive
        # infinity. As d_ij -> +oo, then (1 / d_ij) -> 0.
        if (geodesicMat[row, col] != "Inf") {
          harmonicSum <- harmonicSum + (1 / geodesicMat[row, col])
        }
      }
    }

    # compute average efficiency
    E <- harmonicSum / (N * (N - 1))
  }

  E
}

# The global efficiency E_glob of a graph G, where the graph has N vertices
# and K edges. The notion of global efficiency of a graph is defined in the
# paper (Latora & Marchiori 2003). See also the function averageEfficiency,
# which defines the average efficiency of G. The measure E_glob is defined
# as
#
# E_glob = E(G) / E(G^{ideal})
#
# where G^{ideal} is the complete graph on N vertices. Thus E_glob is a
# ratio of the average efficiencies of two types of graphs: (1) the average
# efficiency of G itself; (2) the average efficiency of the complete graph
# on N vertices, which is the number of vertices in G.
#
# INPUT:
#    geodesicMat -- the matrix of the shortest path lengths between pairs of
#       vertices in G. If i and j are vertices of G, then d_ij denotes the
#       shortest path length between i and j. If geodesicMat describes the
#       geodesics of pairs of vertices in an undirected graph, then
#       geodesicMat is symmetric about the main diagonal.
#
# OUTPUT:
#    the global efficiency E_glob of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
globalEfficiency <- function(geodesicMat) {
  # the number of vertices in geodesicMat
  N <- dim(geodesicMat)[1]

  # compute the average efficiency
  aveEfficiency <- averageEfficiency(geodesicMat)

  # Construct the adjacency matrix of a complete graph on N vertices. By
  # definition of complete graphs, a complete graph K_n and its geodesic
  # matrix G_dist are equivalent. That is, K_n and G_dist are copies of
  # each other. Also, G_dist has 1 everywhere, and 0 on the main diagonal.
  gIdealMat <- matrix(nrow = N, ncol = N)
  gIdealMat[,] <- 1
  for (i in 1:N) {
    gIdealMat[i, i] <- 0
  }

  # compute the average efficiency of the complete graph
  aveEfficiencyGIdeal <- averageEfficiency(gIdealMat)

  # compute the global efficiency
  Eglob <- aveEfficiency / aveEfficiencyGIdeal
}

# The adjacency matrix of G_i. If G is an undirected graph and i is a
# vertex of G, then G_i is the subgraph of neighbours of i, excluding i
# itself.
#
# INPUT:
#    aMat -- the adjacency matrix of the graph G.
#    i -- the index of the vertex whose neighbours we want to consider.
#       Let r and c be the row and column dimensions of aMat, respectively.
#       Then 1 < i < r or 1 < i < c.
#
# OUTPUT:
#    the adjacency matrix of G_i.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
neighboursAdjMat <- function(aMat, i) {
  # the row dimension of aMat
  ## rowNum <- dim(aMat)[1]
  # the column dimension of aMat
  colNum <- dim(aMat)[2]

  # find indices of the immediate neighbours of vertex i
  neighIndex <- c()
  for (col in 1:colNum) {
    if (aMat[i, col] == 1) {
      neighIndex <- c(neighIndex, col)
    }
  }

  # Adjacency matrix of neighbours of i, i.e. the adjacency matrix of G_i
  # in the notation of the paper (Latora & Marchiori 2003).
  neighAMat <- matrix(nrow = length(neighIndex),
                      ncol = length(neighIndex))
  neighAMat[,] <- 0
  for (row in 1:length(neighIndex)) {
    for (col in row:length(neighIndex)) {
      if (aMat[neighIndex[row], neighIndex[col]] == 1) {
        neighAMat[row, col] <- 1
        neighAMat[col, row] <- 1
      }
    }
  }

  neighAMat
}

# The local efficiency E_loc of a graph G, where the graph has N vertices
# and K edges. The notion of local efficiency of a graph is defined in the
# paper (Latora & Marchiori 2003). See also the function averageEfficiency,
# which defines the average efficiency of G. The measure E_loc is defined
# as
#
# E_loc = (1/N) \sum_{i \in G} E(G_i) / E(G^{ideal}_i)
#
# where G_i is the subgraph of neighbours of vertex i, and G^{ideal}_i is
# the complete graph on N_i, which is the number of vertices in G_i. Note
# that G_i excludes the vertex i, and only considers the graph formed by
# its immediate neighbours.
#
# INPUT:
#    aMat -- the adjacency matrix of the graph G. If G is an undirected
#       graph, then aMat is symmetric about the main diagonal.
#
# OUTPUT:
#    the local efficiency E_loc of the graph G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
localEfficiency <- function(aMat) {
  # The number of vertices in the underlying graph G. Thus the column and
  # row dimensions must be equal.
  N <- dim(aMat)[1]

  # summing the ratios (EGi / EIdealGi) for all vertices i
  cumSum <- 0           # the cumulative sum
  EGi <- 0              # average efficiency of G_i
  EIdealGi <- 0         # average efficiency of G^{ideal}_i
  geodesicGi <- 0       # geodesic matrix of G_i
  geodesicIdealGi <- 0  # geodesic matrix of G^{ideal}_i
  idealGi <- 0          # adjacency matrix of G^{ideal}_i
  for (i in 1:N) {
    geodesicGi <- geodist(neighboursAdjMat(aMat, i))$gdist
    EGi <- averageEfficiency(geodesicGi)

    # Construct the adjacency matrix of a complete graph on K_i vertices. By
    # definition of complete graphs, a complete graph K_n and its geodesic
    # matrix G_dist are equivalent. That is, K_n and G_dist are copies of
    # each other. Also, G_dist has 1 everywhere, and 0 on the main diagonal.
    idealGi <- matrix(nrow = dim(geodesicGi)[1], ncol = dim(geodesicGi)[2])
    idealGi[,] <- 1
    for (j in 1:dim(idealGi)[1]) {
      idealGi[j, j] <- 0
    }

    geodesicIdealGi <- geodist(idealGi)$gdist
    EIdealGi <- averageEfficiency(geodesicIdealGi)

    # Prevent division by zero, which is possible when EIdealGi = 0. If
    # both EGi and EIdealGi are zero, then we get (0 / 0), which returns
    # a NaN for "not a number". Caution: we need to consider four cases:
    #
    # EGi  EIdealGi
    # -------------
    # 0    0       <- (EGi / EIdealGi) = 0
    # 0    y1      <- (EGi / EIdealGi) = 0
    # x1   0       <- Is it possible to get this case?
    # x2   y2      <- (EGi / EIdealGi) \in RR\{0}
    #
    # where x1, x2, y1, y2 \in RR
    if ((EGi == 0)) {
      cumSum <- cumSum + 0
    }
    else {
      cumSum <- cumSum + (EGi / EIdealGi)
    }
  }

  Eloc <- cumSum / N
}

# The cost of a network G with N vertices and K edges. For now, we assume
# that G is an undirected graph so that its adjacency matrix is symmetric
# about the main diagonal. The generalization of the Watts-Strogatz model
# contained in (Latora & Marchiori 2003) considers directed as well as
# undirected graphs.
#
# INPUT:
#    adjMat -- the adjacency matrix of G. This adjacency matrix must have
#       the same dimensions as the matrix of distances of G.
#    distMat -- the matrix of distances between pairs of vertices. This
#       distance matrix has the same dimensions as the adjacency matrix of
#       G.
#    gamma -- the cost evaluator function, default is "gamma = WS" for the
#       Watts-Strogatz model. TODO: define further models here apart from
#       Watts-Strogatz.
#
# OUTPUT:
#    the cost of the network G.
#
# AUTHOR:
#    Minh Van Nguyen <nguyenminh2@gmail.com>
#
networkCost <- function(adjMat, distMat, gamma = "WS") {
  netCost <- 0
  if (gamma == "WS") {
    N <- dim(adjMat)[1]  # the number of vertices
    K <- 0               # the number of edges

    # For an undirected graph G with adjacency matrix adjMat, both G and
    # adjMat are symmetric about the main diagonal. Hence we need only
    # consider either the upper triangular or lower triangular matrices,
    # excluding entries along the main diagonal, in counting the number of
    # edges in G. On the other hand, we can also sum the entries in adjMat
    # and divide the result by 2 to get the number of edges in g.
    K <- sum(adjMat) / 2

    netCost <- (2 * K) / (N * (N - 1))
  }

  netCost
}

# The main routine. This is where the network metrics are calculated. For
# the Watts-Strogatz model, the network metrics is comprised of the
# characteristic path length L and the clustering coefficient C. As regards
# the generalization of Watts-Strogatz contained in the paper
# (Latora & Marchiori 2003), the network metrics are the local efficiency
# E_loc, the global efficiency E_glob, and the network cost C.
#
# INPUT:
#   regmat -- a regular matrix
#   probabilities -- a set of re-wiring probabilities
#
# OUTPUT:
#
# AUTHOR:
#     Minh Van Nguyen <nguyenminh2@gmail.com>
#
calculateNetworks <- function(regmat, probabilities) {
  # A set of adjacency matrices.
  nets <- array(NA, dim = c(length(probabilities) + 1, n, n))

  # First matrix is the regular matrix.
  nets[1,,] <- regmat[,]

  # Re-wire with probability p.
  # Put this in a function.
  counter <- 1
  for (p in probabilities) {
    reWiredMat <- array(0, dim = dim(regmat))
    while(connectedness(reWiredMat) < 1) {
      reWiredMat <- reWire(regmat, p)
      print(c("Re-wiring with probability ", p))
    }
    nets[counter + 1,,] <- reWiredMat[,]
    # plot(network(reWiredMat, directed=FALSE),
    #      displaylabels=TRUE, mode="circle")
    # par(ask=TRUE)
    counter <- counter + 1
  }

  # This section is for the Latora-Marchiori generalization.
  # Global and local efficiencies
  Eglob <- NULL
  Eloc <- NULL
  for (counter in 1:(length(probabilities) + 1)) {
    Eglob[counter] <- globalEfficiency(geodist(nets[counter,,])$gdist)
    Eloc[counter] <- localEfficiency(nets[counter,,])
  }

  # structure to return
  result <- cbind(Eglob[1] / Eglob, Eloc / Eloc[1])
}


# ===================================================================
# 2. MAIN SCRIPT
# ===================================================================

# connectivity matrix of a regular network with no loops
regularMatrix <- matrix(0, nrow = n, ncol = n)
for (i in 1:n) {
  for (index in (k / 2):1) {
    # Get right the indexes.
    jplus  <- returnIndex(i + index, n)
    jminus <- returnIndex(i - index, n)
    regularMatrix[i, jplus] <- regularMatrix[i, jminus] <- 1
  }
}
dimnames(regularMatrix)[[2]] <- paste("node", (1:n), sep = "-")
dimnames(regularMatrix)[[1]] <- paste("node", (1:n), sep = "-")

# logarithmically-scaled probability vector
factor <- (maxProb / minProb)^(1 / (numberOfPoints - 1))
probs <- NULL
for (pt in 1:numberOfPoints) {
  probs[pt] <- minProb * factor^(pt - 1)
}

# Call main routine here timesToRepeat times.
# variable aliases
GE <- 1    # global efficiency
LE <- 2    # local efficiency

# Remember, the first probability is zero, i.e. the regular matrix.
results <- array(NA, dim = c(timesToRepeat, length(probs) + 1, 2))
dimnames(results)[[3]] <- c("L / LRef", "C / CRef")
dimnames(results)[[2]] <- paste("prob", c(0, probs), sep = "-")
dimnames(results)[[1]] <- paste("experiment", (1:timesToRepeat), sep = "-")
for (experiment in 1:timesToRepeat) {
  print(c("Executing experiment ", experiment))
  results[experiment,,] <- calculateNetworks(regularMatrix, probs)
}

# averages
GEmeans <- colSums(results[,,GE]) / timesToRepeat
LEmeans <- colSums(results[,,LE]) / timesToRepeat

# Write table of results
summary <- cbind(c(0, probs), GEmeans, LEmeans)
write.table(format(summary, digits = 6, nsmall = 4, justify = "left"),
            file = paste(outputDir, summaryFileName, sep = ""), sep = "\t")

# plot variables
xdata <- probs
# All except the first value, where prob = 0.
ydata <- cbind(GEmeans[-1], LEmeans[-1])
matplot(xdata, ydata, log = "x",
        main = "Global efficiency and local efficiency",
        xlab = "prob",
        ylab = "Eglob[ref] / Eglob and Eloc / Eloc[ref]",
        type = "b")
